/*Copyright (c) Dr. Stephen Ware and the University of New Orleans. All rights reserved.
Contributors: Alireza Shirvani, Edward T. Garcia
Licensed under the Non-Profit Open Software License version 3.0. See LICENSE file in the project root for full license information.*/
using Library;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;

namespace Correlator
{
    public class BindingElement
    {
        public BindingArgument source { get;private set; }

        public TargetBindingArgument target { get;private set; }
        
        public BindingElement(TextBlock text)
        {
            this.source = new BindingArgument(text);
        }

        public void SetTarget(TextBlock targetText)
        {
            target = new TargetBindingArgument(targetText);
        }

        internal void Color()
        {
            target.background = source.background;
        }
    }

    public class BindingArgument
    {
        protected Argument argument;

        public int index
        {
            get
            {
                return text.GetIndex();
            }
        }

        public TextBlock text { get; private set; }

        public SolidColorBrush background
        {
            get { return (SolidColorBrush)text.Background; }
            set { text.Background = value; }
        }

        public Argument data
        {
            get
            {
                return argument ?? 
                    (argument = (text.GetListViewItem().DataContext as ColoredArgument).argument);
            }
        }

        public ListView list
        {
            get { return text.GetListView(); }
        }

        public BindingArgument(TextBlock text)
        {
            this.text = text;
        }
    }

    public class TargetBindingArgument : BindingArgument
    {
        public UAction action { get { return (list.DataContext as ColoredUAction).uaction; } }

        public TargetBindingArgument(TextBlock text) : base(text) { }
    }
}


