/*Copyright (c) Dr. Stephen Ware and the University of New Orleans. All rights reserved.
Contributors: Alireza Shirvani, Edward T. Garcia
Licensed under the Non-Profit Open Software License version 3.0. See LICENSE file in the project root for full license information.*/
using Library;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Linq;

namespace Correlator
{
    /// <summary>
    /// Interaction logic for BindingList.xaml
    /// </summary>
    public partial class BindingList : UserControl
    {
        TypeCollection collection;

        public ObservableCollection<ColoredUAction> actions;

        public delegate void ActionEventDelegate(UAction action);
        public event ActionEventDelegate OnActionDrop,OnActionDelete;

        public delegate void ReOrderEventDelegate(IEnumerable<UAction> uactions);
        public event ReOrderEventDelegate OnReOrder;

        public BindingList()
        {
            InitializeComponent();
            actions = new ObservableCollection<ColoredUAction>();
            list.ItemsSource = actions;
        }

        public void Set(TypeCollection collection)
        {
            this.collection = collection;

        }
        #region DragDrop
        private void list_DragEnter(object sender, DragEventArgs e)
        {
            if (!e.Data.GetDataPresent("Action") || sender == e.Source)
            {
                e.Effects = DragDropEffects.None;
            }
        }

        private void ListView_DragEnter(object sender, DragEventArgs e)
        {
            if (!e.Data.GetDataPresent("Argument") || sender == e.Source)
            {
                e.Effects = DragDropEffects.None;
            }
        }

        #endregion

        private void list_Drop(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent("Action"))
            {
                var action = new ColoredUAction(e.Data.GetData("Action") as MicroAction);
                actions.Add(action);
                OnActionDrop?.Invoke(action.uaction);
                if (actions.Count == 1)
                    action.primary = true;
            }
        }
        
        private void Unbind(object sender, RoutedEventArgs e)
        {

        }

        internal void Load(ColoredAction macro, List<UAction> steps) {
            actions.Clear();
            
            foreach (var item in steps)
            {
                actions.Add(new ColoredUAction(macro, item));
            }
        }

        private void Delete(object sender, RoutedEventArgs e)
        {
            var action = (e.OriginalSource as MenuItem).GetListViewItem().DataContext as ColoredUAction;
            OnActionDelete?.Invoke(action.uaction);
            actions.Remove(action);
        }

        private void ListView_Drop(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent("Argument"))

                if ((e.OriginalSource as TextBlock).Sibling(1)
                    .GetValue(VisibilityProperty).Equals(Visibility.Hidden))
                {
                    var binding = e.Data.GetData("Argument") as BindingElement;
                    binding.SetTarget(e.OriginalSource as TextBlock);
                    if (binding.source.data.Matches(
                        binding.target.data))
                    {
                        binding.Color();
                        binding.target.action[binding.target.index].Binding(
                            binding.source.index);
                    }
                    else
                        MessageBox.Show("Types do not match", "Error");
                }
                else
                    MessageBox.Show("If you want to change the argument type, you must delete the action.");


        }

        internal void Load(string input, List<UAction> steps)
        {
            actions.Clear();

            foreach (var item in steps)
            {
                actions.Add(new ColoredUAction(new ColoredAction(input), item));
            }
        }

        private void Primary_Checked(object sender, RoutedEventArgs e)
        {
            foreach (ColoredUAction item in list.Items)
            {
                if (!item.Equals(((e.OriginalSource as CheckBox).GetListViewItem()).DataContext as ColoredUAction))
                {
                    item.primary = false;
                }
            }
            list.Items.Refresh();
        }

        private void Constant_Click(object sender, RoutedEventArgs e)
        {
            var combo=(e.OriginalSource as Button).Sibling(1) as ComboBox;
            combo.Visibility = Visibility.Visible;
            combo.SelectedIndex = 0;
            (e.OriginalSource as Button).Visibility = Visibility.Hidden;
            combo.Sibling(0).SetValue(TextBlock.BackgroundProperty, Brushes.LightGreen);
            (combo.GetListViewItem().DataContext as ColoredUAction)?
                .uaction[combo.GetIndex()].Binding(combo.SelectedValue.ToString());
        }

        private void Push_Up(object sender, RoutedEventArgs e)
        {
            var index = (e.OriginalSource as Button).GetIndex();
            if (index > 0)
            {
                var current = actions[index];
                actions.Remove(current);
                actions.Insert(index - 1, current);
                OnReOrder?.Invoke(actions.ToList().Select(z => z.uaction));
            }
        }

        private void Push_Down(object sender, RoutedEventArgs e)
        {
            var index = (e.OriginalSource as Button).GetIndex();
            if (index < actions.Count - 1)
            {
                var current = actions[index];
                actions.Insert(index + 2, current);
                actions.RemoveAt(index);
                OnReOrder?.Invoke(actions.ToList().Select(z=>z.uaction));
            }
        }

        private void ConstantSelected(object sender, SelectionChangedEventArgs e)
        {
            var combo = e.OriginalSource as ComboBox;
            if (combo.SelectedIndex == -1) combo.SelectedIndex = 0;
            (combo.GetListView().DataContext as ColoredUAction)?
                .uaction[combo.GetIndex()].Binding(combo.SelectedValue.ToString());
        }

        private void Primary_Unchecked(object sender, RoutedEventArgs e)
        {
            (e.OriginalSource as CheckBox).IsChecked = true;
        }
    }
}


