/*Copyright (c) Dr. Stephen Ware and the University of New Orleans. All rights reserved.
Contributors: Alireza Shirvani, Edward T. Garcia
Licensed under the Non-Profit Open Software License version 3.0. See LICENSE file in the project root for full license information.*/
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;

namespace Correlator
{
    public static class FrameworkElementExtension
    {
        public static ListViewItem GetListViewItem(this FrameworkElement control)
        {
            FrameworkElement parent = control.TemplatedParent as FrameworkElement;
            do
            {
                parent = parent.TemplatedParent as FrameworkElement;
            } while (!(parent is ListViewItem));
            return parent as ListViewItem;
        }

        public static DependencyObject Sibling(this FrameworkElement control,int index)
        {
            return VisualTreeHelper.GetChild(VisualTreeHelper.GetParent(control), index);
        }

        public static ListView GetListView(this ListViewItem item)
        {
            return ItemsControl.ItemsControlFromItemContainer(item) as ListView;
        }

        public static ListView GetListView(this FrameworkElement control)
        {
            return ItemsControl.ItemsControlFromItemContainer(control.GetListViewItem()) as ListView;
        }

        public static int GetIndex(this FrameworkElement control)
        {
            return GetListViewItem(control).GetListView()
            .ItemContainerGenerator.IndexFromContainer(control.GetListViewItem());
        }

        public static List<T> ToList<T>(this ObservableCollection<T> collection)
        {
            var list = new List<T>();
            foreach (var item in collection)
            {
                list.Add(item);
            }
            return list;
        }
    }
}


