/*Copyright (c) Dr. Stephen Ware and the University of New Orleans. All rights reserved.
Contributors: Alireza Shirvani, Edward T. Garcia
Licensed under the Non-Profit Open Software License version 3.0. See LICENSE file in the project root for full license information.*/
using Library;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Linq;
using System.Windows.Media;
using System.Collections.ObjectModel;

namespace Correlator
{
    /// <summary>
    /// Interaction logic for InputWindow.xaml
    /// </summary>
    public partial class InputWindow : Window
    {
        Sequence<string> current;
        List<Sequence<string>> sequences;
        public InputWindow(TypeCollection collection, IEnumerable<Library.Action> actions, string input_path)
        {
            InitializeComponent();
            Setup();
            Initialize(actions, input_path);
            bindings.Set(collection);
        }

        private Sequence<string> GetSequence(string input)
        {
            return sequences.FirstOrDefault(z => z.header.Equals(input));
        }

        private void Setup()
        {
            sequences = new List<Sequence<string>>();
            inputs.onSelection += MacroAction_SelectionChanged;
            bindings.OnActionDrop += Sequence_Add;
            bindings.OnActionDelete += Sequence_Remove;
            bindings.OnReOrder += Bindings_OnReOrder;
        }

        private void Bindings_OnReOrder(IEnumerable<UAction> uactions)
        {
            current.steps = uactions.ToList();
        }

        private void Sequence_Remove(UAction action)
        {
            if (inputs.list.SelectedIndex!=-1)
            {
                var sequence = GetSequence(inputs.list.SelectedValue.ToString());
                if (sequence == null)
                    MessageBox.Show("You must select a domain action from the left");
                else
                    sequence.steps.Remove(action);
            }
        }

        private void Sequence_Add(UAction uaction)
        {
            if (inputs.list.SelectedIndex!=-1)
            {
                var sequence = GetSequence(inputs.list.SelectedValue.ToString());
                if (sequence == null)
                    MessageBox.Show("You must select a domain action from the left");
                else
                    sequence.Add(uaction);
            }
        }

        private void MacroAction_SelectionChanged(string input)
        {
            if (inputs.list.SelectedIndex != -1)
            {
                current = GetSequence(input);
                if (current == null)
                {
                    sequences.Add(current = new Sequence<string>(input));
                }
                bindings.Load(input, current.steps);
            }
        }

        private void Initialize(IEnumerable<Action> actions, string bindings)
        {
            this.actions.list.ItemsSource = actions;
            var reader = new JsonReader();
            sequences = reader.ParseInput(bindings, actions)??new List<Sequence<string>>();
            this.inputs.DataContext = new ObservableCollection<string>
                (sequences.Select(z => z.header).ToList());
        }

        private void Save(object sender, RoutedEventArgs e)
        {
            if (sequences.Any(z => z.steps.Any(x => !x.bound)))
                MessageBox.Show("You must bind all arguments", "Error");
            else
            {
                new JsonWriter().ParseInput("input_bindings.json", sequences);
                MessageBox.Show("Saved to Json", "Success");
            }
        }
    }
}


