/*Copyright (c) Dr. Stephen Ware and the University of New Orleans. All rights reserved.
Contributors: Alireza Shirvani, Edward T. Garcia
Licensed under the Non-Profit Open Software License version 3.0. See LICENSE file in the project root for full license information.*/
using SimpleJSON;
using System.Collections.Generic;
using System.Linq;
using System.IO;
using Library;

namespace Correlator
{
    public class JsonReader
    {
        private JSONNode CheckFile(string path)
        {
            JSONNode config;
            if (File.Exists(path))
            {
                if (!path.EndsWith(".json"))
                    throw new System.FormatException("The selected file is not in json format.");
                config = JSON.Parse(File.ReadAllText(path));
            }
            else
                throw new System.InvalidOperationException("The specified file does not exist.");
            return config;
        }

        public List<Sequence<MacroAction>> Parse(string path, IEnumerable<Action> operators,
            IEnumerable<Action> actions, out string player)
        {
            return Parse(CheckFile(path), operators, actions, out player);
        }

        private List<Sequence<MacroAction>> Parse(JSONNode node, IEnumerable<Action> operators,
            IEnumerable<Action> actions, out string player)
        {
            if (node != null)
            {
                var sequences = new List<Sequence<MacroAction>>();
                player = node.AsObject["player"];
                foreach (JSONNode op in node.AsObject["bindings"].AsArray)
                {
                    if (operators.Any(z => z.name.Equals(op["operator"].Value)))
                    {
                        sequences.Add(new Sequence<MacroAction>((MacroAction)operators.Single(x => x.name.Equals(op["operator"]))));
                        AddActions(op, sequences.Last(), actions);
                    }
                }
                return sequences;
            }
            player = null;
            return new List<Sequence<MacroAction>>();
        }

        private void AddActions<T>(JSONNode op, Sequence<T> sequence, IEnumerable<Action> actions)
        {
            foreach (JSONNode action in op["actions"].AsArray)
            {
                if (actions.Any(z => z.name.Equals(action["name"].Value)))
                {
                    sequence.Add(new UAction((MicroAction)actions.Single(x => x.name.Equals(action["name"].Value))));
                    AddBindings(action["arguments"], sequence.steps.Last(),
                        (sequence.header is MacroAction) ?
                        (sequence.header as MacroAction).name : sequence.header.ToString());
                }
            }
        }

        private void AddBindings(JSONNode action, UAction uAction, MacroAction op)
        {
            for (int i = 0; i < System.Math.Min(uAction.action.args.Count(), action.AsArray.Count); i++)
            {
                int value = 0;
                if (int.TryParse(action.AsArray[i].Value, out value))
                {
                    if (value < op.args.Count() &&
                        op.args.ElementAt(value)
                        .Matches(uAction[i].argument))
                        uAction[i].Binding(value);
                }
                else
                {
                    uAction[i].Binding(action.AsArray[i].Value);
                }
            }
        }

        private void AddBindings(JSONNode action, UAction uAction, string op)
        {
            for (int i = 0; i < System.Math.Min(uAction.action.args.Count(), action.AsArray.Count); i++)
                uAction[i].Binding(action.AsArray[i].Value);
        }

        internal List<Sequence<string>> ParseInput(string bindings, IEnumerable<Library.Action> actions)
        {
            var node = CheckFile(bindings);
            if (node != null)
            {
                var sequences = new List<Sequence<string>>();
                foreach (JSONNode op in node.AsObject["bindings"].AsArray)
                {
                    sequences.Add(new Sequence<string>(op["operator"].Value));
                    AddActions(op, sequences.Last(), actions);
                }
                return sequences;
            }
            return new List<Sequence<string>>();
        }
    }
}


