/*Copyright (c) Dr. Stephen Ware and the University of New Orleans. All rights reserved.
Contributors: Alireza Shirvani, Edward T. Garcia
Licensed under the Non-Profit Open Software License version 3.0. See LICENSE file in the project root for full license information.*/
using Library;
using SimpleJSON;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace Correlator
{
    class JsonWriter
    {
        internal void Parse(string path, List<Sequence<MacroAction>> sequences,string player)
        {
            JSONNode root = new JSONObject();
            root["player"] = player;
            JSONNode node = new JSONArray();
            foreach (var sequence in sequences)
            {
                node.Add(CreateNode(sequence));
            }
            root["bindings"] = node;
            Create(path, root);
        }

        private JSONNode CreateNode<T>(Sequence<T> sequence)
        {
            JSONNode node = new JSONObject();
            node.Add("operator", ((sequence.header is MacroAction)?
                (sequence.header as MacroAction).name:sequence.header.ToString()));
            var list = new JSONArray();
            foreach (var uaction in sequence.steps)
            {
                var action = new JSONObject();
                action.Add("name", uaction.action.name);
                if (uaction.primary) action.Add("primary", 1);
                action.Add("arguments", Bindings(uaction));
                list.Add("action", action);
            }
            node.Add("actions", list);
            return node;
        }

        private JSONNode Bindings(UAction uaction)
        {
            var node = new JSONArray();
            foreach (var binding in uaction.boundArguments)
            {
                node.Add(binding.binding.ToString());
            }
            return node;
        }

        private void Create(string path,JSONNode node)
        {
            if (!File.Exists(path))
                File.Create(path).Close();
            File.WriteAllText(path,node.ToString());
        }

        internal void ParseInput(string path, List<Sequence<string>> sequences)
        {
            JSONNode root = new JSONObject();
            JSONNode node = new JSONArray();
            foreach (var sequence in sequences)
            {
                node.Add(CreateNode(sequence));
            }
            root["bindings"] = node;
            Create(path, root);
        }
    }
}


