/*Copyright (c) Dr. Stephen Ware and the University of New Orleans. All rights reserved.
Contributors: Alireza Shirvani, Edward T. Garcia
Licensed under the Non-Profit Open Software License version 3.0. See LICENSE file in the project root for full license information.*/
using Library;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;

namespace Correlator
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        Sequence<MacroAction> current;
        ListViewItem current_item;
        public List<Sequence<MacroAction>> sequences { get; set; }

        public MainWindow(TypeCollection collection, IEnumerable<Action> operators, IEnumerable<Action> actions,string binding_path,List<string> agents)
        {
            InitializeComponent();
            Setup();
            Initialize(operators,actions,binding_path,agents);
            bindings.Set(collection);
        }

        private Sequence<MacroAction> GetSequence(MacroAction action)
        {
            return sequences.FirstOrDefault(z => z.header.Equals(action));
        }

        private void Setup()
        {
            sequences = new List<Sequence<MacroAction>>();
            operators.list.SelectionChanged += MacroAction_SelectionChanged;
            bindings.OnActionDrop += Sequence_Add;
            bindings.OnActionDelete += Sequence_Remove;
            bindings.OnReOrder += Bindings_OnReOrder;
        }

        private void Bindings_OnReOrder(IEnumerable<UAction> uactions)
        {
            current.steps = uactions.ToList();
        }

        private void Sequence_Remove(UAction action)
        {
            if (operators.list.SelectedIndex != -1)
            {
                var sequence = GetSequence((operators.list.SelectedValue as ColoredAction).action as MacroAction);
                if (sequence == null)
                    MessageBox.Show("You must select a domain action from the left");
                else
                    sequence.steps.Remove(action);
            }
        }

        private void Sequence_Add(UAction uaction)
        {
            if (operators.list.SelectedIndex != -1)
            {
                var sequence = GetSequence((operators.list.SelectedValue as ColoredAction).action as MacroAction);
                if (sequence == null)
                    MessageBox.Show("You must select a domain action from the left");
                else
                    sequence.Add(uaction);
            }
        }
        
        private void MacroAction_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (operators.list.SelectedIndex != -1)
            {
                current_item = operators.list.ItemContainerGenerator.ContainerFromItem(
                    operators.list.SelectedValue) as ListViewItem;
                foreach (var data in operators.list.Items)
                {
                    var item = operators.list.ItemContainerGenerator.ContainerFromItem(data) as ListViewItem;
                    if (!item.Equals(current_item))
                        (VisualTreeHelper.GetChild(item,0) as FrameworkElement).IsEnabled = false;
                }
                (VisualTreeHelper.GetChild(current_item, 0) as FrameworkElement).IsEnabled = true;
                current = GetSequence((operators.list.SelectedValue as ColoredAction).action as MacroAction);
                if (current == null)
                {
                    sequences.Add(current = new Sequence<MacroAction>((operators.list.SelectedValue as ColoredAction)
                        .action as MacroAction));
                }
                bindings.Load(operators.list.SelectedValue as ColoredAction, current.steps);
            }
        }

        private void Initialize(IEnumerable<Action> operators, IEnumerable<Action> actions,
            string bindings,List<string> agents)
        {
            this.operators.list.ItemsSource = operators.Select(z=>new ColoredAction(z));
            this.actions.list.ItemsSource = actions;
            this.operators.agents.ItemsSource = agents;
            var reader2 = new JsonReader();
            string player;
            sequences = reader2.Parse(bindings,operators, actions,out player);
            this.operators.agents.SelectedItem = player;

        }

        private void Save(object sender, RoutedEventArgs e)
        {
            if (sequences.Any(z => z.steps.Any(x => !x.bound)))
                MessageBox.Show("You must bind all arguments", "Error");
            else { 
                new JsonWriter().Parse("bindings.json", sequences,operators.agents.SelectedValue.ToString());
                MessageBox.Show("Saved to Json","Success");
            }
        }
    }
}


