/*Copyright (c) Dr. Stephen Ware and the University of New Orleans. All rights reserved.
Contributors: Alireza Shirvani, Edward T. Garcia
Licensed under the Non-Profit Open Software License version 3.0. See LICENSE file in the project root for full license information.*/
using System.IO;
using System.Linq;
using System.Windows;
using Library;
using System.Windows.Forms;
using System;

namespace Correlator
{
    /// <summary>
    /// Interaction logic for Master.xaml
    /// </summary>
    public partial class Master : Window
    {
        MainWindow window;
        InputWindow input;
        public Master()
        {
            InitializeComponent();
        }

        private void Manage_Bindings1(object sender, RoutedEventArgs e)
        {
            if (window == null)
            {
                var domainTranslator = new DomainTranslator(domain.Text);
                var dlist = domainTranslator.GetActions();
                if (dlist != null)
                {
                    var ulist = new UnityActionTranslator(unity.Text).GetActions();
                    if (ulist != null)
                    {
                        var blist = CheckBindings();
                        if (blist != null)
                        {
                            var alist = domainTranslator.GetAgents(agent.Text).ToList();
                            var clist = domainTranslator.GetTypes();
                            window = new MainWindow(clist, dlist, ulist, blist,alist);
                            window.Closed += (s, a) => { window = null; };
                            window.Show();
                        }
                    }
                }
            }
        }

        private void Manage_Bindings2(object sender, RoutedEventArgs e)
        {
            if (input == null)
            {
                var ulist = new UnityActionTranslator(unity.Text).GetActions();
                if (ulist != null)
                {
                    var blist = CheckBindings2();
                    if (blist != null)
                    {
                        var domainTranslator = new DomainTranslator(domain.Text);
                        var clist = domainTranslator.GetTypes();
                        input = new InputWindow(clist, ulist, blist);
                        input.Closed += (s, a) => { input = null; };
                        input.Show();
                    }
                }
            }
        }

        private string CheckBindings()
        {
            if (!File.Exists(bindings.Text))
                return null;
            return bindings.Text;
        }

        private string CheckBindings2()
        {
            if (!File.Exists("input_" + bindings.Text))
                return null;
            return "input_" + bindings.Text;
        }

        private void Domain(object sender, RoutedEventArgs e)
        {
            domain.Text = FileDialog(".domain", "Domain File (.domain)|*.domain"); ;
        }

        private void Actions(object sender, RoutedEventArgs e)
        {
            unity.Text = FolderDialog();
        }

        private void Bindings(object sender, RoutedEventArgs e)
        {
            bindings.Text = FileDialog(".json", "Bindings JSON File (.json)|*.json");
        }

        string FileDialog(string extension,string filter)
        {
            var dlg = new OpenFileDialog();
            dlg.DefaultExt = extension;
            dlg.Filter = filter;
            if (dlg.ShowDialog()==System.Windows.Forms.DialogResult.OK)
                return dlg.FileName;
            return null;
        }

        string FolderDialog()
        {
            var dlg = new FolderBrowserDialog();
            if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                return dlg.SelectedPath;
            return null;
        }
    }
}


