/*Copyright (c) Dr. Stephen Ware and the University of New Orleans. All rights reserved.
Contributors: Alireza Shirvani, Edward T. Garcia
Licensed under the Non-Profit Open Software License version 3.0. See LICENSE file in the project root for full license information.*/
using Library;
using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;

namespace Correlator
{
    /// <summary>
    /// Interaction logic for OperatorList.xaml
    /// </summary>
    public partial class OperatorList : UserControl
    {
        Point startPoint;
        
        public OperatorList()
        {
            InitializeComponent();
        }

        public string playerConstant
        {
            get
            {
                return agents.SelectedValue.ToString();
            }
        }

        #region DragDrop


        private void ListView_PreviewMouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            startPoint = e.GetPosition(null);
            list.SelectedItem = null;
        }

        private void ListView_DragLeave(object sender, DragEventArgs e)
        {
            if (!e.Data.GetDataPresent("Argument"))
            {
                e.Effects = DragDropEffects.None;
            }
        }

        private void ListView_PreviewMouseMove(object sender, MouseEventArgs e)
        {
                // Get the current mouse position
                Point mousePos = e.GetPosition(null);
                Vector diff = startPoint - mousePos;

                if (e.LeftButton == MouseButtonState.Pressed &&
                    (Math.Abs(diff.X) > SystemParameters.MinimumHorizontalDragDistance ||
                    Math.Abs(diff.Y) > SystemParameters.MinimumVerticalDragDistance))
                {

                    // Find the data behind the ListViewItem
                    if (e.OriginalSource is TextBlock)
                    {
                        var binding = new BindingElement(e.OriginalSource as TextBlock);
                        // Initialize the drag & drop operation
                        DataObject dragData = new DataObject("Argument", binding);
                        DragDrop.DoDragDrop(binding.source.list, dragData, DragDropEffects.Move);
                    }
                }
        }

        #endregion


    }
}


