/*Copyright (c) Dr. Stephen Ware and the University of New Orleans. All rights reserved.
Contributors: Alireza Shirvani, Edward T. Garcia
Licensed under the Non-Profit Open Software License version 3.0. See LICENSE file in the project root for full license information.*/
using SimpleJSON;
using System.IO;
using System.Linq;

namespace ExperienceManager
{
    public class JsonReader
    {
        internal Sequencer Parse(string path, out string player)
        {
            return Parse(CheckFile(path), out player);
        }

        Sequencer Parse(JSONNode node,out string player)
        {
            player = node.AsObject["player"];
            var sequencer = new Sequencer();
            foreach (JSONNode item in node.AsObject["bindings"].AsArray)
            {
                sequencer.Add(new Operator()
                {
                    name = item["operator"].Value,
                    actions = item["actions"].AsArray.Select<Action,JSONObject>(y => new Action(y["name"].Value, y["arguments"].AsArray
                       .Select<int,JSONNumber>(z => int.Parse(z.Value)).ToList(), y["primary"] != null)).ToList()
                });
            }
            return sequencer;
        }

        private JSONNode CheckFile(string path)
        {
            JSONNode config;
            if (File.Exists(path))
            {
                if (!path.EndsWith(".json"))
                    throw new System.FormatException("The selected file is not in json format.");
                config = JSON.Parse(File.ReadAllText(path));
            }
            else
                throw new System.InvalidOperationException("The specified file does not exist.");
            return config;
        }
    }
}


