/*Copyright (c) Dr. Stephen Ware and the University of New Orleans. All rights reserved.
Contributors: Alireza Shirvani, Edward T. Garcia
Licensed under the Non-Profit Open Software License version 3.0. See LICENSE file in the project root for full license information.*/
using PlotGraph;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace ExperienceManager
{
    class Program
    {
        const int max_runs = 10;
        static int runs = 0;
        static string plotpath = @"G:\CamelotPlotgraphs\random\random_temporal.csv";
        static string mappath = @"G:\CamelotPlotgraphs\random\random_actions.txt";
        static string pospath = @"G:\CamelotPlotgraphs\random\random_positions.csv";

        //static string plotpath = @"G:\CamelotPlotgraphs\pruned\pruned_temporal.csv";
        //static string mappath = @"G:\CamelotPlotgraphs\pruned\pruned_actions.txt";
        //static string pospath = @"G:\CamelotPlotgraphs\pruned\pruned_positions.csv";

        static string log = "log";
        static string id = "0";
        static Graph graph;
        static Thread thread;
        /// <summary>
        /// 
        /// </summary>
        /// <param name="args">
        /// Arg0 = The path to the plotgraph file
        /// Arg1 = The path to the action labels file
        /// Arg2 = The path to the positions file
        /// Arg3 = The name for the log files to create (control - experiment)
        /// Arg4 = The participant id
        /// </param>
        static void Main(string[] args)
        {
#if DEBUG
#else
            System.AppDomain.CurrentDomain.UnhandledException += UnhandledExceptionTrapper;
#endif
            if (args.Length > 0)
            {
                plotpath = args[0];
            }
            if (args.Length > 1)
            {
                mappath = args[1];
            }
            if (args.Length > 2)
            {
                pospath = args[2];
            }
            if (args.Length > 3)
            {
                log = args[3];
            }
            if (args.Length > 4)
            {
                id = args[4];
            }
            if (!File.Exists(plotpath))
                Console.WriteLine("Plotgraph file does not exist at " + plotpath);
            else if (!File.Exists(mappath))
                Console.WriteLine("The action labels file does not exist at " + mappath);
            //else if (!File.Exists(pospath))
            //    Console.WriteLine("The positions file does not exist at " + pospath);
            else
            {
                thread = new Thread(RunAsync);
                thread.Start();
            }
            Console.ReadLine();
        }

        internal static void Reset()
        {
            var temp=thread;
            if (++runs >= max_runs)
                Environment.Exit(1);
            else
            {
                Console.WriteLine("@resetting");
                thread = new Thread(RunAsync);
                thread.Start();
                temp.Abort();
            }
        }

        static void RunAsync()
        {
            var database = new Medium.Database();
            var time = DateTime.Now;
            if (graph == null)
                graph = new Graph(plotpath, mappath, pospath, log, id);
            else
                graph.Reset(log, id);
            var exp = new ExperienceManager(graph, database);
            try
            {
                exp.Run("Player");
            }
            catch (ThreadAbortException) { }
            catch (Exception exception)
            {
                var msg = string.Format("@Exception occured in the experience manager:{0}. #StackTrace:#{1}. InnerException: {2}",
                    exception.Message, exception.StackTrace, exception.InnerException?.ToString());
                //Console.WriteLine(msg);
                File.AppendAllText("errorlog.txt", msg);
                Console.WriteLine("start SetNarration('Congratulations! You broke the game! Click X to restart!')");
                Console.WriteLine("start ShowNarration()");
                while (Console.ReadLine() != "input Key Cancel") { }
                Console.WriteLine("start Quit()");
                while (Console.ReadLine() != "input Reset") { }
                runs--;
                Reset();
            }
        }

        static void UnhandledExceptionTrapper(object sender, UnhandledExceptionEventArgs e)
        {
            if (e.ExceptionObject is PlotGraph.PlotGraphException)
                Console.WriteLine((e.ExceptionObject as Exception).Message);
            else
            {
                var exception = (Exception)e.ExceptionObject;
                var msg = string.Format("@Exception occured in the experience manager:{0}. #StackTrace:#{1}. InnerException: {2}",
                    exception.Message, exception.StackTrace, exception.InnerException?.ToString());
                //Console.WriteLine(msg);
                File.AppendAllText("errorlog.txt", msg);
                Console.WriteLine("start SetNarration('Congratulations! You broke the game! Click X to restart!')");
                Console.WriteLine("start ShowNarration()");
                while(Console.ReadLine()!="input Key Cancel") { }
                Console.WriteLine("start Quit()");
                Console.WriteLine("Press Enter to continue");
            }
            Console.ReadLine();
            Environment.Exit(1);
            Environment.Exit(1);
        }
    }
}


