/*Copyright (c) Dr. Stephen Ware and the University of New Orleans. All rights reserved.
Contributors: Alireza Shirvani, Edward T. Garcia
Licensed under the Non-Profit Open Software License version 3.0. See LICENSE file in the project root for full license information.*/
using Library;
using System.Collections.Generic;
using System.Linq;
using System.Text.RegularExpressions;

namespace ExperienceManager
{
    public class Sequencer
    {
        List<Operator> sequences;

        public Sequencer()
        {
            sequences = new List<Operator>();
        }

        internal void Add(Operator step)
        {
            sequences.Add(step);
        }
        
        internal bool? IsPrimaryAction(string step,string action)
        {
            return sequences.FirstOrDefault(z => z.name.Equals(step))?.primary.Equals(action);
        }

        internal IEnumerable<string> Fetch(string msg, out int primary)
        {
            var parameters = Regex.Matches(msg, @"([A-Za-z0-9]+|'[^']*')");
            var arguments = new List<string>();
            for (int i = 1; i < parameters.Count; i++)
            {
                arguments.Add(parameters[i].Value);
            }
            return Fetch(parameters[0].Value, arguments, out primary);
        }

        IEnumerable<string> Fetch(string name, List<string> args, out int primary)
        {
            var step = sequences.FirstOrDefault(z => z.name.Equals(name));
            if (step == null)
            {
                primary = -1;
                return null;
            }
            primary = step.primaryIndex;
            return step.actions.Select(action => string.Format("{0}({1})", action.name,
                action.args.Select(arg => args[arg]).Aggregate((i, j) => string.Format("{0},{1}", i, j))));
        }
    }
}


