/*Copyright (c) Dr. Stephen Ware and the University of New Orleans. All rights reserved.
Contributors: Alireza Shirvani, Edward T. Garcia
Licensed under the Non-Profit Open Software License version 3.0. See LICENSE file in the project root for full license information.*/
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text.RegularExpressions;

namespace Library
{
    public class DomainTranslator : Translator
    {
        public DomainTranslator(string path) : base(path) { }

        public IEnumerable<string> GetAgents(string agent)
        {
            if (collection != null)
            {
                var lines = File.ReadAllLines(path).Where(z =>
                collection[agent].Children.Any(x => z.StartsWith(x.name))).ToList();
                foreach (var line in lines)
                {
                    yield return line.Substring(line.IndexOf(":") + 1, 
                        line.IndexOf(";")- 1 - line.IndexOf(":")).Trim();
                }
            }
        }

        protected override IEnumerable<Action> Actions()
        {
            if (!File.Exists(path))
                yield break;
            var lines = File.ReadAllLines(path).Where(z => z.StartsWith("action") || z.StartsWith("axiom")).ToList();
            foreach (var line in lines)
            {
                var parameters = Regex.Matches(line, @"([A-Za-z0-9_]+)");
                if (parameters.Count < 2)
                    yield break;
                var arguments = new List<Argument>();
                for (int i = 2; i < parameters.Count; i += 2)
                {
                    arguments.Add(new Argument(collection[parameters[i].Value],
                        parameters[i+1].Value));
                }
                yield return new MacroAction(parameters[1].Value, arguments);
            }
        }
    }
}


