/*Copyright (c) Dr. Stephen Ware and the University of New Orleans. All rights reserved.
Contributors: Alireza Shirvani, Edward T. Garcia
Licensed under the Non-Profit Open Software License version 3.0. See LICENSE file in the project root for full license information.*/
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;


namespace Library
{
    public abstract class Translator
    {
        protected string path;
        public Translator(string path)
        {
            this.path = path;
        }

        public List<Action> GetActions()
        {
            if (collection == null)
                return null;
            return Actions().ToList();

        }

        protected abstract IEnumerable<Action> Actions();

        static TypeCollection _collection;
        protected TypeCollection collection
        {
            get
            {
                return _collection ?? (_collection = GetTypes());
            }
        }

        public TypeCollection GetTypes()
        {
            var collection = new TypeCollection();
            var all_lines = File.ReadAllLines(path).ToList();
            var lines = all_lines.Where(z => z.StartsWith("type")).ToList();
            foreach (var line in lines)
            {
                var words = line.Replace(",", string.Empty).Replace(";", string.Empty).Split(' ');
                collection.Add(words[1]);
                collection[words[1]].Add(all_lines.Where(z => z.StartsWith(words[1])&&!z.Contains('('))
                    .Select(z => z.Substring(z.IndexOf(':') + 1, z.Length - 2 - z.IndexOf(':'))));
            }
            foreach (var line in lines)
            {
                var words = line.Replace(",", string.Empty).Replace(";", string.Empty).Split(' ');
                for (int i = 3; i < words.Length; i++)
                {
                    collection.Add(words[i]);
                    if (collection[words[i]] == null)
                        return null;
                    collection[words[i]].Add(collection[words[1]]);
                }
            }
            return collection;
        }
    }
}


