/*Copyright (c) Dr. Stephen Ware and the University of New Orleans. All rights reserved.
Contributors: Alireza Shirvani, Edward T. Garcia
Licensed under the Non-Profit Open Software License version 3.0. See LICENSE file in the project root for full license information.*/
using System;
using System.Collections.Generic;
using System.Linq;

namespace Library
{
    public class Type
    {
        List<Type> children;

        public List<string> constants { get; private set; }
        public string name { get; private set; }

        public List<string> extended_constants
        {
            get
            {
                return constants.Union(children.SelectMany(z => z.constants).ToList()).ToList();
            }
        }
        public Type(string name)
        {
            this.name = name;
            this.children = new List<Type>();
            this.constants = new List<string>();
        }

        public List<Type> Children
        {
            get
            {
                return children;
            }
        }

        public Type this[int index]
        {
            get { return children[index]; }
        }

        public bool Add(Type type)
        {
            if (children.Any(z => z.name.Equals(type.name)))
                return false;
            children.Add(type);
            return true;
        }

        public bool Matches(Type type)
        {
            return this.name.ToLower().Equals(type.name.ToLower()) || children.Any(z => z.Matches(type));
        }

        public override string ToString()
        {
            return name;
        }

        internal void Add(IEnumerable<string> constants)
        {
            this.constants = constants.ToList();
        }
    }
}


