/*Copyright (c) Dr. Stephen Ware and the University of New Orleans. All rights reserved.
Contributors: Alireza Shirvani, Edward T. Garcia
Licensed under the Non-Profit Open Software License version 3.0. See LICENSE file in the project root for full license information.*/
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace Medium
{
    public class ActionSequences:List<Sequence>
    {
        Database database;

        public new void Add(Sequence sequence)
        {
            sequence.database = database;
            base.Add(sequence);
        }

        public ActionSequences(Database database)
        {
            this.database = database;
        }

        internal Sequence Get(string input)
        {
            Sequence sequence = null;
            if (input.Contains("input"))
                sequence= GetInput(input);
            else
                sequence= GetAction(input);
            sequence?.Reset();
            return sequence;
        }

        Sequence GetInput(string input)
        {
            var captures = Regex.Match(input, @"(?<b>input\s(?:arrived|exited)\s\S+\sposition\s\S+)|(?<b>input\s(?:Selected|Key)\s\S+)|(?<b>input\s(?:Cancel))|(?<b>input\s\S+)\s(?:(?<a>\S+)\s*)+");

            var args = GetArgs(captures.Groups["a"].Captures);
            return this.FirstOrDefault(z => z.Satisfied(database,args.ToList())&& captures.Groups["b"]!=null&&captures.Groups["b"].Captures.Count>0
            && z.Matches(captures.Groups["b"].Captures[0].Value))?.Ground(args);
        }

        Sequence GetAction(string input)
        {
            var captures = Regex.Matches(input, @"[^\(,\)]+");
            if (captures.Count > 0)
            {
                var name = captures[0].Value;
                var args = GetArgs(captures);
                return this.FirstOrDefault(z => z.Satisfied(database,args.ToList()) && z.Matches(name))?.Ground(args);
            }
            return null;
        }

        private IEnumerable<string> GetArgs(MatchCollection captures)
        {
            for (int i = 1; i < captures.Count; i++)
            {
                yield return captures[i].Value.Trim();
            }
        }

        private IEnumerable<string> GetArgs(CaptureCollection captures)
        {
            for (int i = 0; i < captures.Count; i++)
            {
                yield return captures[i].Value.Trim();
            }
        }
    }
}


