/*Copyright (c) Dr. Stephen Ware and the University of New Orleans. All rights reserved.
Contributors: Alireza Shirvani, Edward T. Garcia
Licensed under the Non-Profit Open Software License version 3.0. See LICENSE file in the project root for full license information.*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace Medium
{
    internal class Bindings : List<Binding>
    {
        Database database;
        public Bindings(Database database)
        {
            this.database = database;
        }
        internal Bindings Add(string key, string value, params Func<Database, string>[] args)
        {
            base.Add(new Binding()
            {
                key = key,
                value = value,
                args = args,
                name= Regex.Match(key.ToLower(), @"(?<=input\s)\S+").Value
            });
            return this;
        }

        internal bool Contains(string key)
        {
            if (key.ToLower().Contains("arrived") || key.ToLower().Contains("exited"))
                return this.Any(z => z.key.ToLower().Equals(key.ToLower()));
            else if (key.ToLower().Contains("selected"))
                return this.Any(z => z.key.ToLower().Equals(key.ToLower()));
            var keyString = Regex.Match(key.ToLower(), @"(?<=input\s)\S+").Value;
            return this.Any(z => z.name.Equals(keyString));
        }

        internal Func<string, bool> this[string key]
        {
            get
            {
                //matching a key with no {} parameters
                var binding = this.SingleOrDefault(z => z.key.ToLower().Equals(key.ToLower()));
                if (binding==null||binding.value.Contains("*"))
                {
                    //process {} parameters
                    if(binding==null)
                        binding = this.First(z => z.name.Equals(Regex.Match(key.ToLower(), @"(?<=input\s)\S+").Value));
                    //the key allows for matching any plotgraph parameters with the matching key
                    if (binding.value.Contains("*"))
                        return (a) => a.StartsWith(binding.value.Replace("*", string.Empty));
                    else
                        return (a) => a.Equals(Process(key, binding).ToLower());
                }
                else
                    return (a)=>a.Equals(binding.value.ToLower());
            }
        }

        private string Process(string key, Binding binding)
        {
            var result = binding.value.ToString();
            var args = Regex.Match(key, @"(?<=input\s\S+\s)(?<args>\S+(?:\s*))+").Groups["args"];
            for (int i = 0; i < args.Captures.Count; i++)
            {
                if (result.Contains("{" + i.ToString() + "}"))
                    result=result.Replace("{" + i.ToString() + "}", args.Captures[i].Value);
            }
            var count = result.Count(z => z.Equals('{'));
            for (int i = 0; i < count; i++)
            {
                if (binding.args.Length <= i)
                    throw new ArgumentOutOfRangeException(string.Format("Not enough arguments have been provided for this binding '{0}'", binding.key));
                result = result.Replace("{" + (i+args.Captures.Count).ToString() + "}", binding.args[i](database));
            }
            return result;
        }
    }

    class Binding
    {
        public string name { get; set; }
        public string key { get; set; }
        public string value { get; set; }
        public Func<Database, string>[] args { get; set; }
    }
}


