/*Copyright (c) Dr. Stephen Ware and the University of New Orleans. All rights reserved.
Contributors: Alireza Shirvani, Edward T. Garcia
Licensed under the Non-Profit Open Software License version 3.0. See LICENSE file in the project root for full license information.*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Medium
{
    internal class Getter<input>: IGetter where input:class
    {
        dynamic function;
        List<Argument> args;

        
        internal Getter(input function, List<Argument> args=null)
        {
            this.function = function;
            this.args = args;
        }

        public List<string> Get(List<string> list, Database database)
        {
            if (function is System.Action<List<string>, Database>)
            {
                function(Arguments(list).ToList(), database);
                return null;
            }
            try
            {
                var result = function(Arguments(list).ToList(), database);
                if (result is string)
                    return new List<string> { result };
                return result;
            }
            catch (ArgumentOutOfRangeException ex)
            {
                throw new Exception("It is possible that you have not specified enough arguments for this function: " + function.Method.ToString(), ex);
            }
        }

        internal IEnumerable<string> Arguments(List<string> list)
        {
            foreach (var arg in args)
            {
                int temp = 0;
                if (int.TryParse(arg.ToString(), out temp))
                    yield return list[temp];
                else
                    yield return arg.ToString();
            }
        }

    }
}


