/*Copyright (c) Dr. Stephen Ware and the University of New Orleans. All rights reserved.
Contributors: Alireza Shirvani, Edward T. Garcia
Licensed under the Non-Profit Open Software License version 3.0. See LICENSE file in the project root for full license information.*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Medium
{
    public class Lists : Dictionary<string, List<Item>>, ValueSet
    {
        public void Add(string subject, Item content)
        {
            if (ContainsKey(subject))
            {
                if (!this[subject].Contains(content))
                    this[subject].Add(content);
            }
            else
                Add(subject, new List<Item>() { content });
        }

        public void Remove(string subject, Item content)
        {
            if (ContainsKey(subject))
                this[subject].Remove(content);
        }

        public bool Contains(string owner, string item)
        {
            if (ContainsKey(owner))
                return this[owner].Any(z => z["name"].Equals(item));
            return false;
        }

        public bool Contains(string owner, string field, string value)
        {
            if (ContainsKey(owner))
                return this[owner].Any(z => z.ContainsKey(field) && z[field].Equals(value));
            return false;
        }

        public bool Contains(string owner, Item content)
        {
            if(ContainsKey(owner))
            return this[owner].Contains(content);
            return false;
        }

        internal List<string> GetAddToListActions(string owner)
        {
            if (!this.ContainsKey(owner))
                return null;
            return this[owner].Select(z => string.Format("AddToList({0}, '{1}')", z["name"], z["description"])).ToList();
        }

        internal List<Item> Get(string owner)
        {
            if (!this.ContainsKey(owner))
                return null;
            return this[owner];
        }

        internal string Get(string field, string owner, string matchingField, string matchingValue)
        {
            var list = this[owner];
            if (list != null)
            {
                var item=list.FirstOrDefault(z => z[matchingField] != null && z[matchingField].Equals(matchingValue));
                if(item!=null)
                return item["name"];
            }
            return "NotFound";
        }
    }
}


