/*Copyright (c) Dr. Stephen Ware and the University of New Orleans. All rights reserved.
Contributors: Alireza Shirvani, Edward T. Garcia
Licensed under the Non-Profit Open Software License version 3.0. See LICENSE file in the project root for full license information.*/
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace PlotGraph
{
    public class NewActionStream : ActionStream
    {
        const bool conversionRequired = false;
        public NewActionStream(string plotgraphPath, string positionPath) : base(plotgraphPath, positionPath)
        {

        }

        private string Path(string plotgraphPath)
        {
            return string.Format("{0}2.csv", plotgraphPath.Substring(0, plotgraphPath.IndexOf(".csv")));
        }

        protected override void PopulatePositions(string plotgraphPath)
        {
            var path = !conversionRequired ? Path(plotgraphPath) : Rewrite(plotgraphPath);
            base.PopulatePositions(plotgraphPath);
        }

        private string Rewrite(string plotgraphPath)
        {
            var newlines = new List<string>();
            var lines = File.ReadAllLines(plotgraphPath);
            long? id = 0;

            var path = Path(plotgraphPath);
            using (var sw = new StreamWriter(File.Open(path, FileMode.Create)))
            {
                for (int i = 0; i < lines.Length; i++)
                {
                    Console.Clear();
                    Console.Write(i.ToString() + "/" + lines.Length);

                    if (id.HasValue && id.Value != GetId(lines[i]))
                    {
                        sw.Write("\r");
                        id = GetId(lines[i]);
                    }
                    if (i > 0)
                        sw.Write("\n");
                    sw.Write(lines[i]);
                }
                sw.Write("\r\n");
            }

            return path;
        }

        private long? GetId(string v)
        {
            return long.Parse(v.Substring(0, v.IndexOf(',')));
        }

        protected override long GetNextFilePosition(int counter, long lastFilePosition, string plotgraphPath)
        {
            if (lastFilePosition == -1)
                return 0;

            const byte NEW_LINE = 0x0A;
            byte BR = Convert.ToByte('\r');
            const int END_OF_FILE = -1;
            using (var file = File.OpenRead(plotgraphPath))
            {
                byte[] buffer = new byte[2048];
                file.Position = lastFilePosition;
                for (int i = 0; true; i++)
                {
                    var readByte = file.ReadByte();
                    if (readByte == END_OF_FILE)
                        return -1;

                    buffer[i] = (byte)readByte;
                    if (i >= 1 && buffer[i] == NEW_LINE && buffer[i - 1] == BR)
                        return lastFilePosition + i +1;
                }
            }
        }

        public override string[] Get(int from)
        {
            const byte NEW_LINE = 0x0A;
            const int END_OF_FILE = -1;
            List<string> strings = new List<string>();
            using (var file = File.OpenRead(plotgraphPath))
            {
                byte[] buffer = new byte[2048];
                file.Position = positions[from];
                byte previousByte = 0;
                for (int i = 0; true; i++)
                {
                    var readByte = file.ReadByte();
                    if (readByte == END_OF_FILE)
                        break;

                    buffer[i] = (byte)readByte;
                    if (previousByte == NEW_LINE && buffer[i] == NEW_LINE)
                        break;
                    if (buffer[i] == NEW_LINE)
                    {
                        if (i > 0)
                            strings.Add(Encoding.ASCII.GetString(buffer, 0, i));

                        previousByte = NEW_LINE;
                        i = -1;
                    }
                    else
                        previousByte = 0;
                }
            }
            return strings.ToArray();
        }
    }
}


