/*Copyright (c) Dr. Stephen Ware and the University of New Orleans. All rights reserved.
Contributors: Alireza Shirvani, Edward T. Garcia
Licensed under the Non-Profit Open Software License version 3.0. See LICENSE file in the project root for full license information.*/
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace PlotGraph
{
    public class OptimalActionStream : ActionStream
    {
        public OptimalActionStream(string plotgraphPath, string positionPath) : base(plotgraphPath, positionPath)
        {

        }

        protected override long GetNextFilePosition(int current, long lastFilePosition, string plotgraphPath)
        {
            if (lastFilePosition == -1)
                return 0;

            const byte NEW_LINE = 0x0A;
            const int END_OF_FILE = -1;
            byte COMMA = Convert.ToByte(',');
            using (var file = File.OpenRead(plotgraphPath))
            {
                byte[] buffer = new byte[2048];
                file.Position = lastFilePosition;
                for (int i = 0; true; i++)
                {
                    var readByte = file.ReadByte();
                    if (readByte == END_OF_FILE)
                        return -1;

                    buffer[i] = (byte)readByte;
                    var temp = Encoding.ASCII.GetString(buffer, i, 1);

                    if (buffer[i] == NEW_LINE)
                    {
                        var position = lastFilePosition + i;
                        var header = new List<byte>();
                        do
                        {
                            i++;
                            var temp2 = file.ReadByte();
                            if (temp2 == END_OF_FILE)
                                return -1;
                            else
                                header.Add((byte)temp2);
                        } while (!header.Last().Equals(COMMA) &&
                        !header.Last().Equals(END_OF_FILE));
                        var headerString = Encoding.ASCII.GetString(header.ToArray(), 0, header.Count - 1);
                        if (long.Parse(headerString) < current)
                            continue;
                        else //if (long.Parse(headerString) == current)
                            return position;
                    }
                }
            }
        }

        public override string[] Get(int from)
        {
            const byte NEW_LINE = 0x0A;
            const int END_OF_FILE = -1;
            List<string> strings = new List<string>();
            using (var file = File.OpenRead(plotgraphPath))
            {
                byte[] buffer = new byte[2048];
                file.Position = positions[from];
                byte previousByte = 0;
                for (int i = 0; true; i++)
                {
                    var readByte = file.ReadByte();
                    if (readByte == END_OF_FILE)
                        break;

                    buffer[i] = (byte)readByte;
                    //if (previousByte == NEW_LINE && buffer[i] == NEW_LINE)
                    //    break;
                    if (buffer[i] == NEW_LINE)
                    {
                        if (i > 0)
                        {
                            var str = Encoding.ASCII.GetString(buffer, 0, i);
                            if (str.Substring(0, str.IndexOf(',')).Equals(from.ToString()))
                                strings.Add(str);
                            else
                                break;
                        }

                        previousByte = NEW_LINE;
                        i = -1;
                    }
                    else
                        previousByte = 0;
                }
            }
            return strings.ToArray();
        }
    }
}


