/*Copyright (c) Dr. Stephen Ware and the University of New Orleans. All rights reserved.
Contributors: Alireza Shirvani, Edward T. Garcia
Licensed under the Non-Profit Open Software License version 3.0. See LICENSE file in the project root for full license information.*/
using that tuple for all references to tuples of that type within 
//		the commands block. 
// Conditions:
//		Currently, the only condition supported is variable equality.
// Commands: 
//		Actions: Game actions to start.
//			The next action will start immediately unless a wait message is provided.
//			Wait messages:
//				wait started	<-- wait until this action has started
//				wait succeeded	<-- wait until this action has succeeded
//				wait <n> 		<-- wait for n seconds
//		Variable Assignments:
//			$someVariable[brand, new, list]	 <-- create a new variable 
//			$someVariable[n] = someValue	 <-- assign value to index in existing variable
//			  ^ use wisely! There may be multiple variables by that name. This will affect
//				all of them!
//		GameOver():
//			Special action that ends the game.

when input Take DiningRoom.Table {
	ClearList();
	forall description[0] = $diningRoomTable[0]
		AddToList(description[0], description[1], description[2]); wait succeeded
	ShowList(DiningRoom.Table, 'Items on the table'); wait succeeded
}

when input arrived Vizzini position path[0] {
	EnableIcon(Open, path[0], 'Open door', true); wait succeeded
}

when input exited Vizzini position path[0] {
	DisableIcon(Open, path[0]); wait succeeded
}

when input arrived Vizzini position Westley & $swordPosition[0] = drawn {
	$reachedWestley[0] = true
	EnableIcon(TalkTo, Westley, 'Talk'); wait succeeded
	EnableIcon(Attack, Westley, 'Attack!'); wait succeeded
}

when input arrived Vizzini position Westley & $swordPosition[0] = sheathed { 
	$reachedWestley[0] = true
	EnableIcon(TalkTo, Westley, 'Talk to Westley', true);
}

when input exited Vizzini position Westley {
	$reachedWestley[0] = false;
	DisableIcon(TalkTo, Westley);
	DisableIcon(Attack, Westley); wait succeeded
}

when input arrived Vizzini position DiningRoom.Table {
	EnableIcon(Take, DiningRoom.Table, 'Take from table', true); wait succeeded
	forall $diningRoomTable[0]
		EnableIcon(Take, $diningRoomTable[0], 'take', true);
}

when input exited Vizzini position DiningRoom.Table {
	DisableIcon(Take, DiningRoom.Table); wait succeeded
	forall $diningRoomTable[0]
		DisableIcon(Take, $diningRoomTable[0]);
}

when input Take poisonedCup[0]{
	HideList();
	$inventory[poisonedCup[0]];
	DisableInput();
	Take(Vizzini, poisonedCup[0]);
	Take(Westley, poisonedCup[1]); wait succeeded
	Drink(Westley, poisonedCup[1]);
	Drink(Vizzini, poisonedCup[0]); wait succeeded
	Die(Vizzini); wait succeeded
	SetNarration('Pity... The glass you chose was poisoned. The battle of wits has ended, and you have lost!'); wait for succeeded
	ShowNarration(); wait 7
	HideNarration();
	Quit();
	GameOver();
}

when input TalkTo Westley {
	SetLeft(Vizzini);
	SetRight(Westley);
	SetDialog('I challenge you to a battle of wits! I have placed a deadly poison in one of the glasses---but which? Choose your glass and I will drink the other. The game ends when you decide, and one of us is dead. [Ok|Okay I will]'); wait for succeeded
	ShowDialog();
}

when input Key Inventory & $inventoryShown[0] = false {
	ClearList(); wait succeeded
	forall description[0] = $inventory[0]
		AddToList(description[0], description[1], description[2]); wait succeeded
	ShowList(Vizzini, 'Inventory'); wait succeeded
	$inventoryShown[0] = true;
}

when input Key Inventory & $inventoryShown[0] = true {
	HideList();
	$inventoryShown[0] = false;
}

when input Selected Ok {
	HideDialog(); wait succeeded
}

when input Cancel {
	$inventoryShown[0] = false
	HideList(); wait succeeded
}

when input Open path[0] {
	DisableInput()
	FadeOut(); wait succeeded
	SetPosition(Vizzini, path[2], path[1]); wait succeeded
	SetCameraMode(path[3]); wait succeeded
	FadeIn(); wait started
	EnableInput()
	EnableIcon(Open, path[1], 'Open door', true)
}

when input Draw Sword & $reachedWestley[0] = true {
	$inventoryShown[0] = false
	HideList()
	DisableIcon(Draw, Sword)
	EnableIcon(Sheathe, Sword, 'Sheathe sword', true); wait succeeded
	$swordPosition[0] = drawn
	Draw(Sword, Vizzini); wait succeeded
	EnableIcon(Attack, Westley)
}

when input Draw Sword {
	$inventoryShown[0] = false
	HideList()
	DisableIcon(Draw, Sword)
	EnableIcon(Sheathe, Sword, 'Sheathe sword', true); wait succeeded
	$swordPosition[0] = drawn
	Draw(Sword, Vizzini)
}

when input Sheathe Sword {
	$inventoryShown[0] = false
	HideList()
	DisableIcon(Attack, Westley)
	DisableIcon(Sheathe, Sword)
	EnableIcon(Draw, Sword, 'Draw sword', true); wait succeeded
	$swordPosition[0] = sheathed
	Sheathe(Sword, Vizzini); wait succeeded
}

when input Attack Westley {
	Attack(Vizzini, Westley); wait succeeded
	SetNarration('That is correct. You win!'); wait succeeded
	ShowNarration(); wait 7
	HideNarration(); wait succeeded
	Quit();
	GameOver(); wait succeeded
}


